export declare type EjectAsyncOptions = {
    verbose?: boolean;
    force?: boolean;
    packageManager?: 'npm' | 'yarn';
};
/**
 * Entry point into the eject process, delegates to other helpers to perform various steps.
 *
 * 1. Verify git is clean
 * 2. Create native projects (ios, android)
 * 3. Install node modules
 * 4. Apply config to native projects
 * 5. Install CocoaPods
 * 6. Log project info
 */
export declare function ejectAsync(projectRoot: string, options?: EjectAsyncOptions): Promise<void>;
/**
 * Merge two gitignore files together and add a generated header.
 *
 * @param targetGitIgnorePath
 * @param sourceGitIgnorePath
 */
export declare function mergeGitIgnoreFiles(targetGitIgnorePath: string, sourceGitIgnorePath: string): string | null;
export declare function stripDashes(s: string): string;
/**
 * Returns true if the input string matches the default expo main field.
 *
 * - ./node_modules/expo/AppEntry
 * - ./node_modules/expo/AppEntry.js
 * - node_modules/expo/AppEntry.js
 * - expo/AppEntry.js
 * - expo/AppEntry
 *
 * @param input package.json main field
 */
export declare function isPkgMainExpoAppEntry(input?: string): boolean;
